@@ -70,7 +70,7 @@ display_about_dialog (BonoboUIComponent *uic,
   }
 
   bubble->aboutbox= gnome_about_new(_("Bubbling Load Monitor"), VERSION,
-				    "Copyright (C) 1999-2004 Johan Walles",
+				    "Copyright (C) 1999-2007 Johan Walles",
 				    _("This applet displays your CPU load as a bubbling liquid.\n"
 				      "This applet comes with ABSOLUTELY NO WARRANTY, "
 				      "see the LICENSE file for details.\n"
@@ -351,7 +351,7 @@ bubble_applet_factory (PanelApplet *applet,
 
 PANEL_APPLET_BONOBO_FACTORY ("OAFIID:GNOME_BubblemonApplet_Factory",
 			     PANEL_TYPE_APPLET,
-			     "bubblemon",
-			     "0",
+			     PACKAGE_NAME,
+			     VERSION,
 			     bubble_applet_factory,
 			     NULL)
